@@ -825,7 +825,7 @@ int process_message(snac *snac, char *msg, char *req)
 
         follower_add(snac, actor);
 
-        snac_log(snac, xs_fmt("New follower %s", actor));
+        snac_log(snac, xs_fmt("new follower %s", actor));
         do_notify = 1;
     }
     else
@@ -929,7 +929,7 @@ int process_message(snac *snac, char *msg, char *req)
             object = xs_dict_get(object, "id");
 
         if (valid_status(timeline_del(snac, object)))
-            snac_log(snac, xs_fmt("New 'Delete' %s %s", actor, object));
+            snac_debug(snac, 1, xs_fmt("new 'Delete' %s %s", actor, object));
         else
             snac_debug(snac, 1, xs_fmt("ignored 'Delete' for unknown object %s", object));
     }
@@ -1094,7 +1094,7 @@ int activitypub_get_handler(d_char *req, char *q_path,
     else
     if (strcmp(p_path, "outbox") == 0) {
         xs *id = xs_fmt("%s/outbox", snac.actor);
-        xs *elems = timeline_simple_list(&snac, "public", 20);
+        xs *elems = timeline_simple_list(&snac, "public", 0, 20);
         xs *list = xs_list_new();
         msg = msg_collection(&snac, id);
         char *p, *v;
