@@ -120,7 +120,7 @@ void user_free(snac *snac)
 }
 
 
-int user_open(snac *snac, char *uid)
+int user_open(snac *snac, const char *uid)
 /* opens a user */
 {
     int ret = 0;
@@ -359,7 +359,7 @@ d_char *index_list(const char *fn, int max)
 }
 
 
-d_char *index_list_desc(const char *fn, int max)
+d_char *index_list_desc(const char *fn, int skip, int show)
 /* returns an index as a list, in reverse order */
 {
     d_char *list = NULL;
@@ -372,9 +372,9 @@ d_char *index_list_desc(const char *fn, int max)
         char line[256];
         list = xs_list_new();
 
-        /* move to the end minus one entry */
-        if (!fseek(f, 0, SEEK_END) && !fseek(f, -33, SEEK_CUR)) {
-            while (n < max && fgets(line, sizeof(line), f) != NULL) {
+        /* move to the end minus one entry (or more, if skipping entries) */
+        if (!fseek(f, 0, SEEK_END) && !fseek(f, (skip + 1) * -33, SEEK_CUR)) {
+            while (n < show && fgets(line, sizeof(line), f) != NULL) {
                 line[32] = '\0';
                 list = xs_list_append(list, line);
                 n++;
@@ -706,7 +706,7 @@ int follower_add(snac *snac, const char *actor)
 {
     int ret = object_user_cache_add(snac, actor, "followers");
 
-    snac_debug(snac, 2, xs_fmt("follower_add %s %s", actor));
+    snac_debug(snac, 2, xs_fmt("follower_add %s", actor));
 
     return ret == -1 ? 500 : 200;
 }
@@ -856,7 +856,7 @@ d_char *timeline_top_level(d_char *list)
 }
 
 
-d_char *timeline_simple_list(snac *snac, const char *idx_name, int max)
+d_char *timeline_simple_list(snac *snac, const char *idx_name, int skip, int show)
 /* returns a timeline (with all entries) */
 {
     int c_max;
@@ -865,19 +865,19 @@ d_char *timeline_simple_list(snac *snac, const char *idx_name, int max)
     c_max = xs_number_get(xs_dict_get(srv_config, "max_timeline_entries"));
 
     /* never more timeline entries than the configured maximum */
-    if (max > c_max)
-        max = c_max;
+    if (show > c_max)
+        show = c_max;
 
     xs *idx = xs_fmt("%s/%s.idx", snac->basedir, idx_name);
 
-    return index_list_desc(idx, max);
+    return index_list_desc(idx, skip, show);
 }
 
 
-d_char *timeline_list(snac *snac, const char *idx_name, int max)
+d_char *timeline_list(snac *snac, const char *idx_name, int skip, int show)
 /* returns a timeline (only top level entries) */
 {
-    xs *list = timeline_simple_list(snac, idx_name, max);
+    xs *list = timeline_simple_list(snac, idx_name, skip, show);
 
     return timeline_top_level(list);
 }
@@ -1099,7 +1099,7 @@ int actor_get(snac *snac, const char *actor, d_char **data)
 /* returns an already downloaded actor */
 {
     int status = 200;
-    char *d;
+    d_char *d;
 
     if (strcmp(actor, snac->actor) == 0) {
         /* this actor */
@@ -1115,6 +1115,8 @@ int actor_get(snac *snac, const char *actor, d_char **data)
 
     if (data)
         *data = d;
+    else
+        d = xs_free(d);
 
     xs *fn = _object_fn(actor);
     double max_time;
