@@ -66,7 +66,7 @@ d_char *html_actor_icon(snac *snac, d_char *os, char *actor, char *date, char *u
 
                 if (n && i) {
                     char *u = xs_dict_get(i, "url");
-                    xs *img = xs_fmt("<img src=\"%s\" style=\"height: 1em\"/>", u);
+                    xs *img = xs_fmt("<img src=\"%s\" style=\"height: 1em\" loading=\"lazy\"/>", u);
 
                     name = xs_replace_i(name, n, img);
                 }
@@ -84,7 +84,8 @@ d_char *html_actor_icon(snac *snac, d_char *os, char *actor, char *date, char *u
         avatar = xs_fmt("data:image/png;base64, %s", susie);
 
     {
-        xs *s1 = xs_fmt("<p><img class=\"snac-avatar\" src=\"%s\" alt=\"\"/>\n", avatar);
+        xs *s1 = xs_fmt("<p><img class=\"snac-avatar\" src=\"%s\" alt=\"\" "
+                        "loading=\"lazy\"/>\n", avatar);
         s = xs_str_cat(s, s1);
     }
 
@@ -277,6 +278,9 @@ d_char *html_top_controls(snac *snac, d_char *s)
         "<p>%s:<br>\n"
         "<textarea name=\"bio\" cols=\"40\" rows=\"4\">%s</textarea></p>\n"
 
+        "<p><input type=\"checkbox\" name=\"cw\" id=\"cw\" %s>\n"
+        "<label for=\"cw\">%s</label></p>\n"
+
         "<p>%s:<br>\n"
         "<input type=\"text\" name=\"email\" value=\"%s\"></p>\n"
 
@@ -299,6 +303,10 @@ d_char *html_top_controls(snac *snac, d_char *s)
     if (xs_is_null(email))
         email = "";
 
+    char *cw = xs_dict_get(snac->config, "cw");
+    if (xs_is_null(cw))
+        cw = "";
+
     xs *s1 = xs_fmt(_tmpl,
         snac->actor,
         L("Sensitive content"),
@@ -320,6 +328,8 @@ d_char *html_top_controls(snac *snac, d_char *s)
         xs_dict_get(snac->config, "avatar"),
         L("Bio"),
         xs_dict_get(snac->config, "bio"),
+        strcmp(cw, "open") == 0 ? "checked" : "",
+        L("Always show sensitive content"),
         L("Email address for notifications"),
         email,
         L("Password (only to change it)"),
@@ -358,6 +368,13 @@ d_char *build_mentions(snac *snac, char *msg)
 
         if (type && strcmp(type, "Mention") == 0 &&
             href && strcmp(href, snac->actor) != 0 && name) {
+            xs *s1 = NULL;
+
+            if (name[0] != '@') {
+                s1 = xs_fmt("@%s", name);
+                name = s1;
+            }
+
             xs *l = xs_split(name, "@");
 
             /* is it a name without a host? */
@@ -382,7 +399,7 @@ d_char *build_mentions(snac *snac, char *msg)
 }
 
 
-d_char *html_entry_controls(snac *snac, d_char *os, char *msg, int num)
+d_char *html_entry_controls(snac *snac, d_char *os, char *msg, const char *md5)
 {
     char *id    = xs_dict_get(msg, "id");
     char *actor = xs_dict_get(msg, "attributedTo");
@@ -390,7 +407,6 @@ d_char *html_entry_controls(snac *snac, d_char *os, char *msg, int num)
     xs *boosts  = object_announces(id);
 
     xs *s   = xs_str_new(NULL);
-    xs *md5 = xs_md5_hex(id, strlen(id));
 
     s = xs_str_cat(s, "<div class=\"snac-controls\">\n");
 
@@ -399,10 +415,10 @@ d_char *html_entry_controls(snac *snac, d_char *os, char *msg, int num)
             "<form method=\"post\" action=\"%s/admin/action\">\n"
             "<input type=\"hidden\" name=\"id\" value=\"%s\">\n"
             "<input type=\"hidden\" name=\"actor\" value=\"%s\">\n"
-            "<input type=\"hidden\" name=\"redir\" value=\"%d_entry\">\n"
+            "<input type=\"hidden\" name=\"redir\" value=\"%s_entry\">\n"
             "\n",
 
-            snac->actor, id, actor, num
+            snac->actor, id, actor, md5
         );
 
         s = xs_str_cat(s, s1);
@@ -413,9 +429,11 @@ d_char *html_entry_controls(snac *snac, d_char *os, char *msg, int num)
         s = html_button(s, "like", L("Like"));
     }
 
-    if (strcmp(actor, snac->actor) == 0 || xs_list_in(boosts, snac->md5) == -1) {
-        /* not already boosted or us; add button */
-        s = html_button(s, "boost", L("Boost"));
+    if (is_msg_public(snac, msg)) {
+        if (strcmp(actor, snac->actor) == 0 || xs_list_in(boosts, snac->md5) == -1) {
+            /* not already boosted or us; add button */
+            s = html_button(s, "boost", L("Boost"));
+        }
     }
 
     if (strcmp(actor, snac->actor) != 0) {
@@ -449,6 +467,7 @@ d_char *html_entry_controls(snac *snac, d_char *os, char *msg, int num)
             "<input type=\"hidden\" name=\"in_reply_to\" value=\"%s\">\n"
             "<p><input type=\"checkbox\" name=\"sensitive\"> %s\n"
             "<p><input type=\"file\" name=\"attach\">\n"
+            "<input type=\"hidden\" name=\"redir\" value=\"%s_entry\">\n"
             "<p><input type=\"submit\" class=\"button\" value=\"%s\">\n"
             "</form><p></div>\n"
             "</details><p>"
@@ -460,6 +479,7 @@ d_char *html_entry_controls(snac *snac, d_char *os, char *msg, int num)
             ct,
             id,
             L("Sensitive content"),
+            md5,
             L("Post")
         );
 
@@ -472,7 +492,7 @@ d_char *html_entry_controls(snac *snac, d_char *os, char *msg, int num)
 }
 
 
-d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, int *num)
+d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, const char *md5)
 {
     char *id    = xs_dict_get(msg, "id");
     char *type  = xs_dict_get(msg, "type");
@@ -494,10 +514,8 @@ d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, int
     else
         s = xs_str_cat(s, "<div>\n");
 
-    if (level == 0) {
-        xs *s1 = xs_fmt("<a name=\"%d_entry\"></a>\n", *num);
-
-        *num = *num + 1;
+    {
+        xs *s1 = xs_fmt("<a name=\"%s_entry\"></a>\n", md5);
 
         s = xs_str_cat(s, s1);
     }
@@ -613,10 +631,12 @@ d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, int
     if (!xs_is_null(v = xs_dict_get(msg, "sensitive")) && xs_type(v) == XSTYPE_TRUE) {
         if (xs_is_null(v = xs_dict_get(msg, "summary")) || *v == '\0')
             v = "...";
-
-        xs *s1 = xs_fmt("<details><summary>%s [%s]</summary>\n", v, L("SENSITIVE CONTENT"));
+        /* only show it when not in the public timeline and the config setting is "open" */
+        char *cw = xs_dict_get(snac->config, "cw");
+        if (xs_is_null(cw) || local)
+            cw = "";
+        xs *s1 = xs_fmt("<details %s><summary>%s [%s]</summary>\n", cw, v, L("SENSITIVE CONTENT"));
         s = xs_str_cat(s, s1);
-
         sensitive = 1;
     }
 
@@ -657,7 +677,8 @@ d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, int
 
                     if (n && i) {
                         char *u = xs_dict_get(i, "url");
-                        xs *img = xs_fmt("<img src=\"%s\" style=\"height: 1em\"/>", u);
+                        xs *img = xs_fmt("<img src=\"%s\" style=\"height: 1em\" "
+                                         "loading=\"lazy\"/>", u);
 
                         c = xs_replace_i(c, n, img);
                     }
@@ -686,7 +707,7 @@ d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, int
                 char *name = xs_dict_get(v, "name");
 
                 if (url != NULL) {
-                    xs *s1 = xs_fmt("<p><img src=\"%s\" alt=\"%s\"/></p>\n",
+                    xs *s1 = xs_fmt("<p><img src=\"%s\" alt=\"%s\" loading=\"lazy\"/></p>\n",
                         url, xs_is_null(name) ? "" : name);
 
                     s = xs_str_cat(s, s1);
@@ -713,7 +734,7 @@ d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, int
     /** controls **/
 
     if (!local)
-        s = html_entry_controls(snac, s, msg, *num);
+        s = html_entry_controls(snac, s, msg, md5);
 
     /** children **/
     xs *children = object_children(id);
@@ -739,7 +760,7 @@ d_char *html_entry(snac *snac, d_char *os, char *msg, int local, int level, int
                 s = xs_str_cat(s, "</details>\n");
 
             if (chd != NULL)
-                s = html_entry(snac, s, chd, local, level + 1, num);
+                s = html_entry(snac, s, chd, local, level + 1, cmd5);
             else
                 snac_debug(snac, 2, xs_fmt("cannot read from timeline child %s", cmd5));
 
@@ -769,13 +790,12 @@ d_char *html_user_footer(snac *snac, d_char *s)
 }
 
 
-d_char *html_timeline(snac *snac, char *list, int local)
+d_char *html_timeline(snac *snac, char *list, int local, int skip, int show, int show_more)
 /* returns the HTML for the timeline */
 {
     d_char *s = xs_str_new(NULL);
     char *v;
     double t = ftime();
-    int num = 0;
 
     s = html_user_header(snac, s, local);
 
@@ -791,7 +811,7 @@ d_char *html_timeline(snac *snac, char *list, int local)
         if (!valid_status(object_get_by_md5(v, &msg, NULL)))
             continue;
 
-        s = html_entry(snac, s, msg, local, 0, &num);
+        s = html_entry(snac, s, msg, local, 0, v);
     }
 
     s = xs_str_cat(s, "</div>\n");
@@ -828,6 +848,15 @@ d_char *html_timeline(snac *snac, char *list, int local)
         s = xs_str_cat(s, s1);
     }
 
+    if (show_more) {
+        xs *s1 = xs_fmt(
+            "<p>"
+            "<a href=\"%s%s?skip=%d&show=%d\" name=\"snac-more\">%s</a>"
+            "</p>\n", snac->actor, local ? "" : "/admin", skip + show, show, L("More..."));
+
+        s = xs_str_cat(s, s1);
+    }
+
     s = xs_str_cat(s, "</body>\n</html>\n");
 
     return s;
@@ -956,6 +985,7 @@ int html_get_handler(d_char *req, char *q_path, char **body, int *b_size, char *
     xs *uid = NULL;
     char *p_path;
     int cache = 1;
+    int save = 1;
     char *v;
 
     xs *l = xs_split_n(q_path, "/", 2);
@@ -993,6 +1023,14 @@ int html_get_handler(d_char *req, char *q_path, char **body, int *b_size, char *
     if ((v = xs_dict_get(srv_config, "disable_cache")) && xs_type(v) == XSTYPE_TRUE)
         cache = 0;
 
+    int skip = 0;
+    int show = xs_number_get(xs_dict_get(srv_config, "max_timeline_entries"));
+    char *q_vars = xs_dict_get(req, "q_vars");
+    if ((v = xs_dict_get(q_vars, "skip")) != NULL)
+        skip = atoi(v), cache = 0, save = 0;
+    if ((v = xs_dict_get(q_vars, "show")) != NULL)
+        show = atoi(v), cache = 0, save = 0;
+
     if (p_path == NULL) {
         /* public timeline */
         xs *h = xs_str_localtime(0, "%Y-%m.html");
@@ -1005,13 +1043,16 @@ int html_get_handler(d_char *req, char *q_path, char **body, int *b_size, char *
             status  = 200;
         }
         else {
-            xs *list = timeline_list(&snac, "public", XS_ALL);
+            xs *list = timeline_list(&snac, "public", skip, show);
+            xs *next = timeline_list(&snac, "public", skip + show, 1);
+
+            *body = html_timeline(&snac, list, 1, skip, show, xs_list_len(next));
 
-            *body   = html_timeline(&snac, list, 1);
             *b_size = strlen(*body);
             status  = 200;
 
-            history_add(&snac, h, *body, *b_size);
+            if (save)
+                history_add(&snac, h, *body, *b_size);
         }
     }
     else
@@ -1031,13 +1072,16 @@ int html_get_handler(d_char *req, char *q_path, char **body, int *b_size, char *
             else {
                 snac_debug(&snac, 1, xs_fmt("building timeline"));
 
-                xs *list = timeline_list(&snac, "private", XS_ALL);
+                xs *list = timeline_list(&snac, "private", skip, show);
+                xs *next = timeline_list(&snac, "private", skip + show, 1);
+
+                *body   = html_timeline(&snac, list, 0, skip, show, xs_list_len(next));
 
-                *body   = html_timeline(&snac, list, 0);
                 *b_size = strlen(*body);
                 status  = 200;
 
-                history_add(&snac, "timeline.html_", *body, *b_size);
+                if (save)
+                    history_add(&snac, "timeline.html_", *body, *b_size);
             }
         }
     }
@@ -1065,7 +1109,7 @@ int html_get_handler(d_char *req, char *q_path, char **body, int *b_size, char *
 
             list = xs_list_append(list, md5);
 
-            *body   = html_timeline(&snac, list, 1);
+            *body   = html_timeline(&snac, list, 1, 0, 0, 0);
             *b_size = strlen(*body);
             status  = 200;
         }
@@ -1098,7 +1142,7 @@ int html_get_handler(d_char *req, char *q_path, char **body, int *b_size, char *
     if (strcmp(p_path, ".rss") == 0) {
         /* public timeline in RSS format */
         d_char *rss;
-        xs *elems = timeline_simple_list(&snac, "public", 20);
+        xs *elems = timeline_simple_list(&snac, "public", 0, 20);
         xs *bio   = not_really_markdown(xs_dict_get(snac.config, "bio"));
         char *p, *v;
 
@@ -1191,7 +1235,7 @@ int html_post_handler(d_char *req, char *q_path, d_char *payload, int p_size,
     uid = xs_list_get(l, 1);
     if (!uid || !user_open(&snac, uid)) {
         /* invalid user */
-        srv_log(xs_fmt("html_get_handler bad user %s", uid));
+        srv_log(xs_fmt("html_post_handler bad user %s", uid));
         return 404;
     }
 
@@ -1379,6 +1423,12 @@ int html_post_handler(d_char *req, char *q_path, d_char *payload, int p_size,
             snac.config = xs_dict_set(snac.config, "avatar", v);
         if ((v = xs_dict_get(p_vars, "bio")) != NULL)
             snac.config = xs_dict_set(snac.config, "bio", v);
+        if ((v = xs_dict_get(p_vars, "cw")) != NULL &&
+            strcmp(v, "on") == 0) {
+            snac.config = xs_dict_set(snac.config, "cw", "open");
+        } else { /* if the checkbox is not set, the parameter is missing */
+            snac.config = xs_dict_set(snac.config, "cw", "");
+        }
         if ((v = xs_dict_get(p_vars, "email")) != NULL)
             snac.config = xs_dict_set(snac.config, "email", v);
 
@@ -1428,4 +1478,3 @@ int html_post_handler(d_char *req, char *q_path, d_char *payload, int p_size,
 
     return status;
 }
-
