@@ -24,21 +24,10 @@ int usage(void)
     printf("queue {basedir} {uid}            Processes a user queue\n");
     printf("follow {basedir} {uid} {actor}   Follows an actor\n");
     printf("unfollow {basedir} {uid} {actor} Unfollows an actor\n");
-
-//    printf("check {basedir} [{uid}]          Checks the database\n");
-
-//    printf("update {basedir} {uid}           Sends a user update to followers\n");
-//    printf("passwd {basedir} {uid}           Sets the password for {uid}\n");
-//    printf("unfollow {basedir} {uid} {actor} Unfollows an actor\n");
-//    printf("mute {basedir} {uid} {actor}     Mutes an actor\n");
-//    printf("unmute {basedir} {uid} {actor}   Unmutes an actor\n");
-//    printf("like {basedir} {uid} {url}       Likes an url\n");
-//    printf("announce {basedir} {uid} {url}   Announces (boosts) an url\n");
-//    printf("note {basedir} {uid} {'text'}    Sends a note to followers\n");
-
     printf("request {basedir} {uid} {url}    Requests an object\n");
     printf("actor {basedir} {uid} {url}      Requests an actor\n");
     printf("note {basedir} {uid} {'text'}    Sends a note to followers\n");
+    printf("resetpwd {basedir} {uid}         Resets the password of a user\n");
 
     return 1;
 }
@@ -150,6 +139,10 @@ int main(int argc, char *argv[])
         return 1;
     }
 
+    if (strcmp(cmd, "resetpwd") == 0) {
+        return resetpwd(&snac);
+    }
+
     if (strcmp(cmd, "queue") == 0) {
         process_queue(&snac);
         return 0;
@@ -166,7 +159,7 @@ int main(int argc, char *argv[])
 #endif
 
         xs *idx  = xs_fmt("%s/private.idx", snac.basedir);
-        xs *list = index_list_desc(idx, 256);
+        xs *list = index_list_desc(idx, 0, 256);
         xs *tl   = timeline_top_level(list);
 
         xs *j    = xs_json_dumps_pp(tl, 4);
