@@ -1,7 +1,7 @@
 /* snac - A simple, minimalistic ActivityPub instance */
 /* copyright (c) 2022 grunfink - MIT license */
 
-#define VERSION "2.13"
+#define VERSION "2.14"
 
 #define USER_AGENT "snac/" VERSION
 
@@ -36,17 +36,17 @@ typedef struct _snac {
     d_char *md5;        /* actor url md5 */
 } snac;
 
-int user_open(snac *snac, char *uid);
+int user_open(snac *snac, const char *uid);
 void user_free(snac *snac);
 d_char *user_list(void);
 
 void snac_debug(snac *snac, int level, d_char *str);
 #define snac_log(snac, str) snac_debug(snac, 0, str)
 
-int validate_uid(char *uid);
+int validate_uid(const char *uid);
 
-d_char *hash_password(char *uid, char *passwd, char *nonce);
-int check_password(char *uid, char *passwd, char *hash);
+d_char *hash_password(const char *uid, const char *passwd, const char *nonce);
+int check_password(const char *uid, const char *passwd, const char *hash);
 
 void srv_archive(char *direction, char *req, char *payload, int p_size,
                  int status, char *headers, char *body, int b_size);
@@ -58,7 +58,7 @@ int index_add(const char *fn, const char *md5);
 int index_del(const char *fn, const char *md5);
 int index_first(const char *fn, char *buf, int size);
 d_char *index_list(const char *fn, int max);
-d_char *index_list_desc(const char *fn, int max);
+d_char *index_list_desc(const char *fn, int skip, int show);
 
 int object_add(const char *id, d_char *obj);
 int object_add_ow(const char *id, d_char *obj);
@@ -85,8 +85,8 @@ d_char *follower_list(snac *snac);
 
 double timeline_mtime(snac *snac);
 int timeline_del(snac *snac, char *id);
-d_char *timeline_simple_list(snac *snac, const char *idx_name, int max);
-d_char *timeline_list(snac *snac, const char *idx_name, int max);
+d_char *timeline_simple_list(snac *snac, const char *idx_name, int skip, int show);
+d_char *timeline_list(snac *snac, const char *idx_name, int skip, int show);
 int timeline_add(snac *snac, char *id, char *o_msg, char *parent, char *referrer);
 void timeline_admire(snac *snac, char *o_msg, char *id, char *admirer, int like);
 
@@ -173,4 +173,5 @@ int html_post_handler(d_char *req, char *q_path, d_char *payload, int p_size,
                       char **body, int *b_size, char **ctype);
 
 int initdb(const char *_basedir);
-int adduser(char *uid);
+int adduser(const char *uid);
+int resetpwd(snac *snac);
